from herre import Herre
from rath.contrib.fakts.links.aiohttp import FaktsAIOHttpLink
from rath.contrib.fakts.links.graphql_ws import FaktsGraphQLWSLink
from rath.contrib.herre.links.auth import HerreAuthLink
from rath.links.split import SplitLink

from arkitekt_next_next.healthz import FaktsHealthz
from fakts import Fakts
from graphql import OperationType
from unlok_next.rath import UnlokLinkComposition, UnlokRath
from unlok_next.unlok import Unlok


class ArkitektNextUnlok(Unlok):
    rath: UnlokRath
    healthz: FaktsHealthz


def build_arkitekt_next_unlok_next(herre: Herre, fakts: Fakts):
    return ArkitektNextUnlok(
        rath=UnlokRath(
            link=UnlokLinkComposition(
                auth=HerreAuthLink(herre=herre),
                split=SplitLink(
                    left=FaktsAIOHttpLink(fakts_group="lok", fakts=fakts),
                    right=FaktsGraphQLWSLink(fakts_group="lok", fakts=fakts),
                    split=lambda o: o.node.operation != OperationType.SUBSCRIPTION,
                ),
            )
        ),
        healthz=FaktsHealthz(fakts_group="lok", fakts=fakts),
    )



import rich_click as click
from importlib import import_module
from arkitekt_next.apps.types import App
from arkitekt_next.cli.commands.run.utils import import_builder
from arkitekt_next.cli.vars import get_console, get_manifest
from arkitekt_next.cli.options import with_builder
import json
import os


async def run_app(app):
    async with app:
        await app.rekuest.run()


@click.command("prod")
@click.pass_context
@click.option(
    "--pretty",
    "-p",
    help="Should we just output json?",
    is_flag=True,
    default=False,
)
@click.option(
    "--machine-readable",
    "-mr",
    help="Should we just output json?",
    is_flag=True,
    default=False,
)
@with_builder
def templates(
    ctx,
    pretty: bool,
    machine_readable: bool,
    builder: str = "arkitekt_next.builders.easy",
):
    """Runs the app in production mode

    \n
    You can specify the builder to use with the --builder flag. By default, the easy builder is used, which is designed to be easy to use and to get started with.

    """
    from rekuest.definition.registry import get_default_definition_registry

    manifest = get_manifest(ctx)
    console = get_console(ctx)

    entrypoint = manifest.entrypoint
    identifier = manifest.identifier
    entrypoint_file = f"{manifest.entrypoint}.py"
    os.path.realpath(entrypoint_file)

    builder_func = import_builder(builder)

    entrypoint = manifest.entrypoint

    with console.status("Loading entrypoint module..."):
        try:
            import_module(entrypoint)
        except ModuleNotFoundError as e:
            console.print(f"Could not find entrypoint module {entrypoint}")
            raise e

    app: App = builder_func(
        identifier=identifier,
        version="dev",
        logo=manifest.logo,
    )

    rekuest = app.services.get("rekuest")
    if rekuest is None:
        console.print("No rekuest service found in app")
        return

    x = {
        key: item.dict(by_alias=True)
        for key, item in rekuest.agent.extensions[
            "default"
        ].definition_registry.templates.items()
    }
    if machine_readable:
        print("--START_TEMPLATES--" + json.dumps(x) + "--END_TEMPLATES--")

    else:
        if pretty:
            console.print(json.dumps(x, indent=2))
        else:
            print(json.dumps(x))



from kabinet.kabinet import Kabinet
from kabinet.rath import KabinetLinkComposition, KabinetRath
from rath.links.split import SplitLink
from rath.contrib.fakts.links.aiohttp import FaktsAIOHttpLink
from rath.contrib.fakts.links.graphql_ws import FaktsGraphQLWSLink
from rath.contrib.herre.links.auth import HerreAuthLink
from graphql import OperationType
from arkitekt_next_next.healthz import FaktsHealthz
from fakts import Fakts
from herre import Herre


class ArkitektNextKabinet(Kabinet):
    rath: KabinetRath
    healthz: FaktsHealthz


def build_arkitekt_next_kabinet(herre: Herre, fakts: Fakts):
    return ArkitektNextKabinet(
        rath=KabinetRath(
            link=KabinetLinkComposition(
                auth=HerreAuthLink(herre=herre),
                split=SplitLink(
                    left=FaktsAIOHttpLink(fakts_group="kabinet", fakts=fakts),
                    right=FaktsGraphQLWSLink(fakts_group="kabinet", fakts=fakts),
                    split=lambda o: o.node.operation != OperationType.SUBSCRIPTION,
                ),
            )
        ),
        healthz=FaktsHealthz(fakts_group="kabinet", fakts=fakts),
    )



"""Qt related modules.

This module contains Modules that are Qt related, and help to integrate
ArkitektNext with Qt applications.

The main component is the MagicBar, which is a widget that can be added
to any Qt application. It will then allow the user to configure and connect
to ArkitektNext, and configure settings.
"""
from .magic_bar import MagicBar
from .builder import build_arkitekt_next_qt_app

__all__ = ["MagicBar", "build_arkitekt_next_qt_app"]